/*--------------------------------------------------------------------------+
$Id: codetemplates.xml 1 2011-01-01 00:00:01Z hoelzl $
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.internal;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.conqat.lib.commons.collections.UnmodifiableList;
import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.extension.ITutorialProvider;
import org.fortiss.tooling.kernel.extension.ITutorialWhitelistProvider;
import org.fortiss.tooling.kernel.extension.data.TutorialStep;
import org.fortiss.tooling.kernel.service.IPersistencyService;
import org.fortiss.tooling.kernel.service.ITutorialService;

/**
 * This class implements the {@link ITutorialService} interface.
 * 
 * @author hoelzl
 * @author $Author: hoelzl $
 * @version $Rev: 18709 $
 * @ConQAT.Rating RED Hash:
 */
public final class TutorialService implements ITutorialService, ITutorialWhitelistProvider {

	/** The collection of all defined tutorials. */
	private List<ITutorialProvider> allTutorials = new ArrayList<>();

	/** The active tutorial. */
	private ITutorialProvider activeTutorial = null;
	/** The active tutorial step. */
	private TutorialStep activeStep = null;

	/** {@inheritDoc} */
	@Override
	public boolean isTutorialActive() {
		return activeTutorial != null;
	}

	/** {@inheritDoc} */
	@Override
	public ITutorialProvider getActiveTutorial() {
		return activeTutorial;
	}

	/** {@inheritDoc} */
	@Override
	public TutorialStep getActiveTutorialStep() {
		return activeStep;
	}

	/** {@inheritDoc} */
	@Override
	public Collection<ITutorialProvider> getAllTutorialProviders() {
		return new UnmodifiableList<>(allTutorials);
	}

	/** {@inheritDoc} */
	@Override
	public void registerTutorial(ITutorialProvider provider) {
		if(!allTutorials.contains(provider)) {
			allTutorials.add(provider);
		}
	}

	/** {@inheritDoc} */
	@Override
	public void startTutorial(ITutorialProvider provider) {
		activeTutorial = provider;
		activeStep = activeTutorial.getInitialStep();

		IPersistencyService.INSTANCE.addDummyEObjectAsTopLevelElement(activeTutorial
				.getRootElement());
		System.out.println("start tutorial for provider: " + activeTutorial.getClass().getName());
	}

	/** {@inheritDoc} */
	@Override
	public void stopTutorial() {
		activeStep = null;
		activeTutorial = null;
	}

	/** {@inheritDoc} */
	@Override
	public boolean displayElementInNavigator(EObject element) {
		if(activeTutorial == null || activeStep == null) {
			return false;
		}
		ITutorialWhitelistProvider global = activeTutorial.getGlobalWhitelistProvider();
		if(global != null && global.displayElementInNavigator(element)) {
			return true;
		}
		ITutorialWhitelistProvider local = activeStep.getWhitelistProvider();
		return local != null && local.displayElementInNavigator(element);
	}
}
